# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.


def pytest_selenium_capture_debug(item, report, extra):
    """ Called when gathering debug information for the HTML report. """


def pytest_selenium_runtest_makereport(item, report, summary, extra):
    """ Called when making the HTML report. """



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import json

from py.xml import html
import pytest
import requests

from pytest_selenium.drivers.cloud import Provider


class SauceLabs(Provider):

    API = "https://saucelabs.com/rest/v1/{username}/jobs/{session}"
    JOB = "https://saucelabs.com/jobs/{session}"

    @property
    def auth(self):
        return self.username, self.key

    @property
    def executor(self):
        return "https://ondemand.saucelabs.com/wd/hub"

    @property
    def username(self):
        return self.get_credential(
            "username", ["SAUCELABS_USERNAME", "SAUCELABS_USR", "SAUCE_USERNAME"]
        )

    @property
    def key(self):
        return self.get_credential(
            "key", ["SAUCELABS_API_KEY", "SAUCELABS_PSW", "SAUCE_ACCESS_KEY"]
        )

    def uses_driver(self, driver):
        return driver.lower() == self.name.lower()


@pytest.mark.optionalhook
def pytest_selenium_capture_debug(item, report, extra):
    provider = SauceLabs()
    if not provider.uses_driver(item.config.getoption("driver")):
        return

    pytest_html = item.config.pluginmanager.getplugin("html")
    extra.append(pytest_html.extras.html(_video_html(item._driver.session_id)))


@pytest.mark.optionalhook
def pytest_selenium_runtest_makereport(item, report, summary, extra):
    provider = SauceLabs()
    if not provider.uses_driver(item.config.getoption("driver")):
        return

    passed = report.passed or (report.failed and hasattr(report, "wasxfail"))
    session_id = item._driver.session_id

    # Add the job URL to the summary
    provider = SauceLabs()
    job_url = get_job_url(item.config, provider, session_id)
    summary.append("{0} Job: {1}".format(provider.name, job_url))
    pytest_html = item.config.pluginmanager.getplugin("html")
    # Add the job URL to the HTML report
    extra.append(pytest_html.extras.url(job_url, "{0} Job".format(provider.name)))

    try:
        # Update the job result
        api_url = provider.API.format(session=session_id, username=provider.username)
        job_info = requests.get(api_url, auth=provider.auth, timeout=10).json()
        if report.when == "setup" or job_info.get("passed") is not False:
            # Only update the result if it's not already marked as failed
            data = json.dumps({"passed": passed})
            requests.put(api_url, data=data, auth=provider.auth, timeout=10)
    except Exception as e:
        summary.append(
            "WARNING: Failed to update {0} job status: {1}".format(provider.name, e)
        )


def driver_kwargs(request, test, capabilities, **kwargs):
    provider = SauceLabs()

    _capabilities = capabilities
    if os.getenv("SAUCELABS_W3C") == "true":
        _capabilities = capabilities.setdefault("sauce:options", {})

    _capabilities.setdefault("username", provider.username)
    _capabilities.setdefault("accessKey", provider.key)
    _capabilities.setdefault("name", test)
    markers = [x.name for x in request.node.iter_markers()]
    tags = _capabilities.get("tags", []) + markers
    if tags:
        _capabilities["tags"] = tags

    kwargs = {
        "command_executor": provider.executor,
        "desired_capabilities": capabilities,
    }
    return kwargs


def _video_html(session):
    flash_vars = 'config={{\
        "clip":{{\
            "url":"https://assets.saucelabs.com/jobs/{session}/video.flv",\
            "provider":"streamer",\
            "autoPlay":false,\
            "autoBuffering":true}},\
        "play": {{\
            "opacity":1,\
            "label":null,\
            "replayLabel":null}},\
        "plugins":{{\
            "streamer":{{\
                "url":"https://cdn1.saucelabs.com/sauce_skin_deprecated\
                /lib/flowplayer/flowplayer.pseudostreaming-3.2.13.swf",\
                "queryString":"%%3Fstart%%3D%%24%%7Bstart%%7D"}},\
            "controls":{{\
                "mute":false,\
                "volume":false,\
                "backgroundColor":"rgba(0,0,0,0.7)"}}}},\
        "playerId":"player{session}",\
        "playlist":[{{\
            "url":"https://assets.saucelabs.com/jobs/{session}/video.flv",\
            "provider":"streamer",\
            "autoPlay":false,\
            "autoBuffering":true}}]}}'.format(
        session=session
    )

    return str(
        html.div(
            html.object(
                html.param(value="true", name="allowfullscreen"),
                html.param(value="always", name="allowscriptaccess"),
                html.param(value="high", name="quality"),
                html.param(value="#000000", name="bgcolor"),
                html.param(value=flash_vars.replace(" ", ""), name="flashvars"),
                width="100%",
                height="100%",
                type="application/x-shockwave-flash",
                data="https://cdn1.saucelabs.com/sauce_skin_deprecated/lib/"
                "flowplayer/flowplayer-3.2.17.swf",
                name="player_api",
                id="player_api",
            ),
            id="player{session}".format(session=session),
            style="border:1px solid #e6e6e6; float:right; height:240px;"
            "margin-left:5px; overflow:hidden; width:320px",
        )
    )


def get_job_url(config, provider, session_id):
    from datetime import datetime

    job_url = provider.JOB.format(session=session_id)
    job_auth = config.getini("saucelabs_job_auth").lower()

    if job_auth == "none":
        return job_url

    if job_auth == "token":
        return get_auth_url(job_url, provider, session_id)
    elif job_auth == "hour":
        time_format = "%Y-%m-%d-%H"
    elif job_auth == "day":
        time_format = "%Y-%m-%d"
    else:
        raise ValueError("Invalid authorization type: {}".format(job_auth))

    ttl = datetime.utcnow().strftime(time_format)
    return get_auth_url(job_url, provider, session_id, ttl)


def get_auth_url(url, provider, session_id, ttl=None):
    import hmac
    from hashlib import md5

    key = "{0.username}:{0.key}".format(provider)
    if ttl:
        key += ":{}".format(ttl)
    token = hmac.new(key.encode("utf-8"), session_id.encode("utf-8"), md5).hexdigest()
    return "{}?auth={}".format(url, token)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import pytest
import requests

from hashlib import md5
from py.xml import html
from pytest_selenium.drivers.cloud import Provider

HOST = "hub.testingbot.com"
PORT = 443


class TestingBot(Provider):

    API = "https://api.testingbot.com/v1/tests/{session}"
    JOB = "http://testingbot.com/members/tests/{session}"

    def __init__(self, host=None, port=None):
        super(TestingBot, self).__init__()
        self.host = host
        self.port = port

    @property
    def auth(self):
        return self.key, self.secret

    @property
    def executor(self):
        return "{1}://{0.host}:{0.port}/wd/hub".format(
            self, "http" if self.host == "localhost" else "https"
        )

    @property
    def key(self):
        return self.get_credential("key", ["TESTINGBOT_KEY", "TESTINGBOT_USR"])

    @property
    def secret(self):
        return self.get_credential("secret", ["TESTINGBOT_SECRET", "TESTINGBOT_PSW"])


@pytest.mark.optionalhook
def pytest_selenium_capture_debug(item, report, extra):
    provider = TestingBot()
    if not provider.uses_driver(item.config.getoption("driver")):
        return

    session_id = item._driver.session_id
    auth_url = get_auth_url(
        "https://testingbot.com/tests/{}.mp4".format(session_id), provider, session_id
    )
    pytest_html = item.config.pluginmanager.getplugin("html")
    extra.append(pytest_html.extras.html(_video_html(auth_url, session_id)))


@pytest.mark.optionalhook
def pytest_selenium_runtest_makereport(item, report, summary, extra):
    provider = TestingBot()
    if not provider.uses_driver(item.config.getoption("driver")):
        return

    passed = report.passed or (report.failed and hasattr(report, "wasxfail"))
    session_id = item._driver.session_id

    # Add the job URL to the summary
    job_url = provider.JOB.format(session=session_id)
    summary.append("{0} Job: {1}".format(provider.name, job_url))
    pytest_html = item.config.pluginmanager.getplugin("html")
    # Add the job URL to the HTML report
    extra.append(pytest_html.extras.url(job_url, "{0} Job".format(provider.name)))

    try:
        # Update the job result
        api_url = provider.API.format(session=session_id)
        job_info = requests.get(api_url, auth=provider.auth, timeout=10).json()
        if report.when == "setup" or job_info.get("success") is not False:
            # Only update the result if it's not already marked as failed
            data = {"test[success]": "1" if passed else "0"}
            requests.put(api_url, data=data, auth=provider.auth, timeout=10)
    except Exception as e:
        summary.append(
            "WARNING: Failed to update {0} job status: {1}".format(provider.name, e)
        )


def driver_kwargs(request, test, capabilities, host, port, **kwargs):
    provider = TestingBot(host, port)

    capabilities.setdefault("name", test)
    capabilities.setdefault("client_key", provider.key)
    capabilities.setdefault("client_secret", provider.secret)
    markers = [x.name for x in request.node.iter_markers()]
    groups = capabilities.get("groups", []) + markers
    if groups:
        capabilities["groups"] = groups
    kwargs = {
        "command_executor": provider.executor,
        "desired_capabilities": capabilities,
    }
    return kwargs


def _video_html(video_url, session):
    return str(
        html.div(
            html.video(
                html.source(src=video_url, type="video/mp4"),
                width="100%",
                height="100%",
                controls="controls",
            ),
            id="mediaplayer{session}".format(session=session),
            style="border:1px solid #e6e6e6; float:right; height:240px;"
            "margin-left:5px; overflow:hidden; width:320px",
        )
    )


def get_auth_url(url, provider, session_id):
    key = "{0.key}:{0.secret}:{1}".format(provider, session_id)
    token = md5(key.encode("utf-8")).hexdigest()
    return "{}?auth={}".format(url, token)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import pytest
import requests

from pytest_selenium.drivers.cloud import Provider


class BrowserStack(Provider):

    API = "https://www.browserstack.com/automate/sessions/{session}.json"

    @property
    def auth(self):
        return self.username, self.key

    @property
    def executor(self):
        return "https://hub.browserstack.com/wd/hub"

    @property
    def username(self):
        return self.get_credential(
            "username", ["BROWSERSTACK_USERNAME", "BROWSERSTACK_USR"]
        )

    @property
    def key(self):
        return self.get_credential(
            "key", ["BROWSERSTACK_ACCESS_KEY", "BROWSERSTACK_PSW"]
        )


@pytest.mark.optionalhook
def pytest_selenium_runtest_makereport(item, report, summary, extra):
    provider = BrowserStack()
    if not provider.uses_driver(item.config.getoption("driver")):
        return

    passed = report.passed or (report.failed and hasattr(report, "wasxfail"))
    session_id = item._driver.session_id
    api_url = provider.API.format(session=session_id)

    try:
        job_info = requests.get(api_url, auth=provider.auth, timeout=10).json()
        job_url = job_info["automation_session"]["browser_url"]
        # Add the job URL to the summary
        summary.append("{0} Job: {1}".format(provider.name, job_url))
        pytest_html = item.config.pluginmanager.getplugin("html")
        # Add the job URL to the HTML report
        extra.append(pytest_html.extras.url(job_url, "{0} Job".format(provider.name)))
    except Exception as e:
        summary.append(
            "WARNING: Failed to determine {0} job URL: {1}".format(provider.name, e)
        )

    try:
        # Update the job result
        job_status = job_info["automation_session"]["status"]
        status = "running" if passed else "error"
        if report.when == "teardown" and passed:
            status = "completed"
        if job_status not in ("error", status):
            # Only update the result if it's not already marked as failed
            requests.put(
                api_url,
                headers={"Content-Type": "application/json"},
                params={"status": status},
                auth=provider.auth,
                timeout=10,
            )
    except Exception as e:
        summary.append("WARNING: Failed to update job status: {0}".format(e))


def driver_kwargs(request, test, capabilities, **kwargs):
    provider = BrowserStack()
    capabilities.setdefault("name", test)
    capabilities.setdefault("browserstack.user", provider.username)
    capabilities.setdefault("browserstack.key", provider.key)
    kwargs = {
        "command_executor": provider.executor,
        "desired_capabilities": capabilities,
    }
    return kwargs



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from distutils.version import LooseVersion
from selenium import __version__ as SELENIUM_VERSION


def driver_kwargs(capabilities, driver_log, driver_path, **kwargs):

    # Selenium 3.14.0 deprecated log_file in favour of service_log_path
    if LooseVersion(SELENIUM_VERSION) < LooseVersion("3.14.0"):
        kwargs = {"log_file": driver_log}
    else:
        kwargs = {"service_log_path": driver_log}

    if capabilities:
        kwargs["capabilities"] = capabilities
    if driver_path is not None:
        kwargs["executable_path"] = driver_path
    return kwargs



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from distutils.version import LooseVersion
import warnings
import logging

import pytest
from selenium import __version__ as SELENIUM_VERSION
from selenium.webdriver import FirefoxProfile
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
from selenium.webdriver.firefox.options import Options

LOGGER = logging.getLogger(__name__)


def pytest_addoption(parser):
    group = parser.getgroup("selenium", "selenium")
    group._addoption(
        "--firefox-path", metavar="path", help="path to the firefox binary."
    )
    group._addoption(
        "--firefox-preference",
        action="append",
        default=[],
        dest="firefox_preferences",
        metavar=("name", "value"),
        nargs=2,
        help="additional firefox preferences.",
    )
    group._addoption(
        "--firefox-profile", metavar="path", help="path to the firefox profile."
    )
    group._addoption(
        "--firefox-extension",
        action="append",
        default=[],
        dest="firefox_extensions",
        metavar="path",
        help="path to a firefox extension.",
    )


def pytest_configure(config):
    config.addinivalue_line(
        "markers",
        "firefox_arguments(args): arguments to be passed to "
        "Firefox. This marker will be ignored for other browsers. For "
        "example: firefox_arguments('-foreground')",
    )
    config.addinivalue_line(
        "markers",
        "firefox_preferences(dict): preferences to be passed to "
        "Firefox. This marker will be ignored for other browsers. For "
        "example: firefox_preferences({'browser.startup.homepage': "
        "'https://pytest.org/'})",
    )


def driver_kwargs(capabilities, driver_log, driver_path, firefox_options, **kwargs):

    # Selenium 3.14.0 deprecated log_path in favour of service_log_path
    if LooseVersion(SELENIUM_VERSION) < LooseVersion("3.14.0"):
        kwargs = {"log_path": driver_log}
    else:
        kwargs = {"service_log_path": driver_log}

    if capabilities:
        kwargs["capabilities"] = capabilities
    if driver_path is not None:
        kwargs["executable_path"] = driver_path

    # Selenium 3.8.0 deprecated firefox_options in favour of options
    if LooseVersion(SELENIUM_VERSION) < LooseVersion("3.8.0"):
        kwargs["firefox_options"] = firefox_options
    else:
        kwargs["options"] = firefox_options
    return kwargs


@pytest.fixture
def firefox_options(request, firefox_path, firefox_profile):
    options = Options()

    if firefox_profile is not None:
        options.profile = firefox_profile

    if firefox_path is not None:
        options.binary = FirefoxBinary(firefox_path)

    for arg in get_arguments_from_markers(request.node):
        options.add_argument(arg)

    for name, value in get_preferences_from_markers(request.node).items():
        options.set_preference(name, value)

    return options


def get_arguments_from_markers(node):
    arguments = []
    for m in node.iter_markers("firefox_arguments"):
        arguments.extend(m.args)
    return arguments


def get_preferences_from_markers(node):
    preferences = dict()
    for mark in node.iter_markers("firefox_preferences"):
        preferences.update(mark.args[0])
    return preferences


@pytest.fixture(scope="session")
def firefox_path(pytestconfig):
    if pytestconfig.getoption("firefox_path"):
        warnings.warn(
            "--firefox-path has been deprecated and will be removed in a "
            "future release. Please make sure the Firefox binary is in the "
            "default location, or the system path. If you want to specify a "
            "binary path then use the firefox_options fixture to and set this "
            "using firefox_options.binary.",
            DeprecationWarning,
        )
        return pytestconfig.getoption("firefox_path")


@pytest.fixture
def firefox_profile(pytestconfig):
    profile = None
    if pytestconfig.getoption("firefox_profile"):
        profile = FirefoxProfile(pytestconfig.getoption("firefox_profile"))
        warnings.warn(
            "--firefox-profile has been deprecated and will be removed in "
            "a future release. Please use the firefox_options fixture to "
            "set a profile path or FirefoxProfile object using "
            "firefox_options.profile.",
            DeprecationWarning,
        )
    if pytestconfig.getoption("firefox_preferences"):
        profile = profile or FirefoxProfile()
        warnings.warn(
            "--firefox-preference has been deprecated and will be removed in "
            "a future release. Please use the firefox_options fixture to set "
            "preferences using firefox_options.set_preference. If you are "
            "using Firefox 47 or earlier then you will need to create a "
            "FirefoxProfile object with preferences and set this using "
            "firefox_options.profile.",
            DeprecationWarning,
        )
        for preference in pytestconfig.getoption("firefox_preferences"):
            name, value = preference
            if value.isdigit():
                # handle integer preferences
                value = int(value)
            elif value.lower() in ["true", "false"]:
                # handle boolean preferences
                value = value.lower() == "true"
            profile.set_preference(name, value)
        profile.update_preferences()
    if pytestconfig.getoption("firefox_extensions"):
        profile = profile or FirefoxProfile()
        warnings.warn(
            "--firefox-extensions has been deprecated and will be removed in "
            "a future release. Please use the firefox_options fixture to "
            "create a FirefoxProfile object with extensions and set this "
            "using firefox_options.profile.",
            DeprecationWarning,
        )
        for extension in pytestconfig.getoption("firefox_extensions"):
            profile.add_extension(extension)
    return profile
